######################################################################
# The Falcon Programming Language.
# CMake module for find_package( falcon )
######################################################################
#
# Generates the following variables.
# - Falcon_INCLUDE_DIRS: List of directories for inclusion
# - Falcon_LIBRARY_DIRS: List of directories for library search
# - Falcon_BINARY_DIR: Where the falcon binary resides
# - Falcon_MODULES_DIR: Default installation directory for modules
#
# - Falcon_LIBRARIES: Libraries needed for link.
#
######################################################################
# Notes: set -DMOD_INSTALL=<target> to have modules installed in a
#        non-default directory.
######################################################################

get_filename_component(SELF_DIR ${CMAKE_CURRENT_LIST_FILE} PATH)

# OPTIONAL for the case this project is built as a subproject.
if(NOT TARGET falcon_engine)
  include(${SELF_DIR}/falcon-core-targets.cmake OPTIONAL)
endif()

set(Falcon_APP_DIR "lib/falcon/apps")
set(Falcon_MOD_DIR "lib/falcon")
set(Falcon_BIN_DIR "bin")
set(Falcon_LIB_DIR "lib")
set(Falcon_MAN_DIR "share/man/man1")
set(Falcon_INC_DIR "include/falcon0.9.6")
set(Falcon_CMAKE_DIR "lib/falcon/cmake")

# Allow flexible installation location to perform out-of-install module builds
# Also, on WIN32, the target installation directory is decided during package install.
string(REGEX REPLACE "[^/]+" ".." _ups "${Falcon_CMAKE_DIR}")
get_filename_component(Falcon_PREFIX ${SELF_DIR}/${_ups} ABSOLUTE)

set(Falcon_INCLUDE_DIR "${Falcon_PREFIX}/include/falcon0.9.6")
set(Falcon_BINARY_DIR "${Falcon_PREFIX}/bin")
set(Falcon_LIBRARY_DIR "${Falcon_PREFIX}/lib")
set(Falcon_MODULES_DIR  "${Falcon_PREFIX}/lib/falcon")

# If defined, prefer the source directory include files over the installed ones
if(DEFINED ENV{FALCON_SRC_TREE})
  list(APPEND Falcon_INCLUDE_DIRS "$ENV{FALCON_SRC_TREE}/core/include")
  message( STATUS "Falcon build configured for DEVELOPMENT BUILD under \"$ENV{FALCON_SRC_TREE}\"" )
else()
  message( STATUS "Falcon build configured for SYSTEM WIDE BUILD" )
endif()

list(APPEND Falcon_INCLUDE_DIRS "${Falcon_INCLUDE_DIR}")

#Versioning
set( Falcon_VERSION "0.9.6.6" )
set( Falcon_VERSION_NAME "Chimera" )
#
set( Falcon_MAJOR_VERSION 0 )
set( Falcon_MINOR_VERSION 9 )
set( Falcon_SUBMINOR_VERSION 6 )
set( Falcon_PATCH_VERSION 6 )

#Check if everything is in place.
find_program(Falcon_EXECUTABLE falcon "${Falcon_BINARY_DIR}")
find_program(Falcon_falpack_EXECUTABLE falpack "${Falcon_BINARY_DIR}")
find_library(Falcon_ENGINE falcon_engine "${Falcon_LIBRARY_DIR}")
find_file(Falcon_ENGINE_INC falcon/engine.h "${Falcon_INCLUDE_DIR}" )

# Prepare Falcon in development environment.
list(APPEND Falcon_INCLUDE_DIRS  ${Falcon_INCLUDE_DIR})
list(APPEND Falcon_LIBRARY_DIRS  ${Falcon_LIBRARY_DIR})

set(Falcon_LIBRARIES falcon_engine)

# for install(TARGETS .. ${FALCON_INSTALL_DESTINATIONS})
set(FALCON_INSTALL_DESTINATIONS
  RUNTIME DESTINATION ${Falcon_BIN_DIR}
  LIBRARY DESTINATION ${Falcon_LIB_DIR}
  ARCHIVE DESTINATION ${Falcon_ARCH_DIR}
)
# NOTE modules are installed via
#   install(FILES .. DESTINATION ${FALCON_MOD_DIR})
# since they are neither RUNTIME, LIBRARY nor ARCHIVE.

if(WIN32)
   #mingw requires -mthreads global option
   if(CMAKE_GENERATOR STREQUAL "MinGW Makefiles")
      list(APPEND CMAKE_EXE_LINKER_FLAGS -mthreads )
      list(APPEND CMAKE_SHARED_LINKER_FLAGS -mthreads )
      list(APPEND CMAKE_MODULE_LINKER_FLAGS -mthreads )
   endif()
endif(WIN32)


########################################################àà
# Falcon CMAKE suite
# Useful functions to finalize modules
#

macro( falcon_define_module varname modname )
      set( ${varname} "${modname}_fm" )
endmacro( falcon_define_module )

function(FALCON_FINALIZE_MODULE tgt )
      target_link_libraries(${tgt} ${Falcon_LIBRARIES} )
      falcon_install_module( ${tgt} )
endfunction(FALCON_FINALIZE_MODULE)

function(FALCON_FINALIZE_MODULE2 tgt libs)
      target_link_libraries(${tgt} ${Falcon_LIBRARIES} ${libs} )
      falcon_install_module( ${tgt} )
endfunction(FALCON_FINALIZE_MODULE2)

function(FALCON_INSTALL_MODULE tgt )
   if(APPLE)
      set_target_properties(${tgt} PROPERTIES
         PREFIX ""
         SUFFIX ".dylib" )
   else()
      set_target_properties(${tgt} PROPERTIES
         PREFIX "" )
   endif()

   set( CMAKE_INSTALL_PREFIX, "/usr" ) 

   if( DEFINED MOD_INSTALL )
      set( dest "${MOD_INSTALL}" )
   else()
      set( dest "${Falcon_MOD_DIR}" )
   endif()
   
   install( TARGETS ${tgt}
            DESTINATION ${dest} )
endfunction(FALCON_INSTALL_MODULE)
